"""Module containing UiPath trigger definitions."""

__all__ = [
    "UiPathResumeTrigger",
    "UiPathResumeTriggerType",
    "UiPathApiTrigger",
    "UiPathResumeTriggerName",
]

from uipath.core.triggers.trigger import (
    UiPathApiTrigger,
    UiPathResumeTrigger,
    UiPathResumeTriggerName,
    UiPathResumeTriggerType,
)
